@@ -117,8 +117,10 @@ sca <- function(y, x, controls, data, weights=NULL,
           }
           else{
             system.time(models <- pblapply(
-              formulae, function(x2) summary(lm(x2, data=data,
-                                                weights=get(weights))), cl=cl))
+              formulae, function(x2){
+                environment(x2) <- environment()
+                summary(lm(x2, data=data, weights=get(weights)))},
+              cl=cl))
           }
         }
         else{
@@ -130,9 +132,10 @@ sca <- function(y, x, controls, data, weights=NULL,
           }
           else{
             system.time(models <- pblapply(
-              formulae, function(x2) summary(
-                glm(x2, data=data, weights=get(weights),
-                    family=eval(parse(text=family_link)))),
+              formulae, function(x2){
+                environment(x2) <- environment()
+                summary(glm(x2, data=data, weights=get(weights),
+                    family=eval(parse(text=family_link))))},
               cl=cl))
           }
         }
@@ -145,8 +148,10 @@ sca <- function(y, x, controls, data, weights=NULL,
           }
           else{
             models <- parLapply(
-              cl, formulae, function(x2) summary(lm(x2, data=data,
-                                                    weights=get(weights))))
+              cl, formulae, function(x2){
+                environment(x2) <- environment()
+                summary(lm(x2, data=data, weights=get(weights)))}
+              )
           }
         }
         else{
@@ -157,9 +162,11 @@ sca <- function(y, x, controls, data, weights=NULL,
           }
           else{
             models <- parLapply(
-              cl, formulae, function(x2) summary(
-                glm(x2, data=data, weights=get(weights),
-                    family=eval(parse(text=family_link)))))
+              cl, formulae, function(x2){
+                environment(x2) <- environment()
+                summary(glm(x2, data=data, weights=get(weights),
+                    family=eval(parse(text=family_link))))}
+              )
           }
         }
       }
@@ -184,9 +191,9 @@ sca <- function(y, x, controls, data, weights=NULL,
         }
         else{
           system.time(models <- pblapply(formulae,
-                                         function(x2) summary(feols(x2,
-                                                                    data=data,
-                                                                    weights=get(weights))),
+                                         function(x2){
+                                           summary(feols(x2, data=data,
+                                                         weights=data[[weights]]))},
                                          cl=cl))
         }
       }
@@ -197,8 +204,10 @@ sca <- function(y, x, controls, data, weights=NULL,
         }
         else{
           models <- parLapply(cl, formulae,
-                              function(x2) summary(feols(x2, data=data,
-                                                         weights=get(weights))))
+                              function(x2){
+                                summary(feols(x2, data=data,
+                                              weights=data[[weights]]))}
+                              )
         }
       }
     }
@@ -218,9 +227,13 @@ sca <- function(y, x, controls, data, weights=NULL,
               formulae, function(x2) summary(lm(x2, data=data))))
           }
           else{
+
             system.time(models <- pblapply(
-              formulae, function(x2) summary(lm(x2, data=data,
-                                                weights=get(weights)))))
+              formulae, function(x2){
+                environment(x2) <- environment()
+                summary(lm(x2, data=data, weights=get(weights)))
+                }
+              ))
           }
         }
         else{
@@ -231,9 +244,12 @@ sca <- function(y, x, controls, data, weights=NULL,
           }
           else{
             system.time(models <- pblapply(
-              formulae, function(x2) summary(
-                glm(x2, data=data, weights=get(weights),
-                    family=eval(parse(text=family_link))))))
+              formulae, function(x2){
+                environment(x2) <- environment()
+                summary(
+                  glm(x2, data=data, weights=get(weights),
+                      family=eval(parse(text=family_link))))}
+              ))
           }
         }
       }
@@ -243,8 +259,10 @@ sca <- function(y, x, controls, data, weights=NULL,
             models <- lapply(formulae, function(x2) summary(lm(x2, data=data)))
           }
           else{
-            models <- lapply(formulae, function(x2) summary(lm(x2, data=data,
-                                                               weights=get(weights))))
+            models <- lapply(formulae, function(x2){
+              environment(x2) <- environment()
+              summary(lm(x2, data=data, weights=get(weights)))}
+              )
           }
         }
         else{
@@ -256,9 +274,11 @@ sca <- function(y, x, controls, data, weights=NULL,
           }
           else{
             models <- lapply(formulae,
-                             function(x2) summary(
-                               glm(x2, data=data, weights=get(weights),
-                                   family=eval(parse(text=family_link)))))
+                             function(x2){
+                               environment(x2) <- environment()
+                               summary(glm(x2, data=data, weights=get(weights),
+                                   family=eval(parse(text=family_link))))}
+                             )
           }
         }
       }
@@ -277,8 +297,9 @@ sca <- function(y, x, controls, data, weights=NULL,
         }
         else{
           system.time(models <- pblapply(
-            X=formulae, function(x2) summary(feols(x2, data=data,
-                                                   weights=get(weights)))))
+            X=formulae, function(x2){
+              summary(feols(x2, data=data, weights=data[[weights]]))}
+            ))
         }
       }
       else{
@@ -286,8 +307,9 @@ sca <- function(y, x, controls, data, weights=NULL,
           models <- lapply(X=formulae, function(x2) summary(feols(x2, data=data)))
         }
         else{
-          models <- lapply(X=formulae, function(x2) summary(feols(x2, data=data,
-                                                                  weights=get(weights))))
+          models <- lapply(X=formulae, function(x2){
+            summary(feols(x2, data=data, weights=data[[weights]]))}
+            )
         }
       }
     }
@@ -949,6 +971,8 @@ plotControlDistributions <- function(sca_data, title="", type="density"){
 #'                fixed effects.
 #' @param data A data frame containing the variables provided in `formula` and
 #'             any clustering variables passed to `cluster`.
+#' @param weights Optional string with the column name in `data` that contains
+#'                weights.
 #' @param types A string or vector of strings specifying what types of
 #'              standard errors are desired. Defaults to "all".
 #'
@@ -1034,7 +1058,8 @@ plotControlDistributions <- function(sca_data, title="", type="density"){
 #' se_compare(formula = "Salnty ~ T_degC + ChlorA", data = bottles,
 #'            types = c("HC0", "HC1", "HC3"))
 #'
-se_compare <- function(formula, data, types="all", cluster=NULL,
+se_compare <- function(formula, data, weights=NULL,
+                       types="all", cluster=NULL,
                        clusteredOnly=FALSE, fixedEffectsOnly=FALSE,
                        bootSamples=NULL, bootSampleSize=NULL){
 
@@ -1050,12 +1075,23 @@ se_compare <- function(formula, data, types="all", cluster=NULL,
   # present and models are estimated with feols() rather than lm()
   if(grepl("|", formula, fixed=T)){
 
-    model_fe <- tryCatch(feols(as.formula(formula), data=data),
-                         error=function(cond){
-                           message("Fixed effects model estimation failed.",
-                                   cond)
-                           return(NULL)
-                         })
+    if(is.null(weights)){
+      model_fe <- tryCatch(feols(as.formula(formula), data=data),
+                           error=function(cond){
+                             message("Fixed effects model estimation failed.",
+                                     cond)
+                             return(NULL)
+                           })
+    }
+    else{
+      model_fe <- tryCatch(
+        feols(as.formula(formula), data=data, weights=data[[weights]]),
+                           error=function(cond){
+                             message("Fixed effects model estimation failed.",
+                                     cond)
+                             return(NULL)
+                           })
+    }
 
       if(is.null(model_fe)){
         message("Fixed effects model estimation failed.")
@@ -1101,9 +1137,17 @@ se_compare <- function(formula, data, types="all", cluster=NULL,
             samples <- bootSamples
             sample_sizes <- bootSampleSize
 
-            boot <- se_boot(data=data, formula=formula, n_x=n_x,
-                            n_samples=bootSamples[[1]],
-                            sample_size=bootSampleSize[[1]])
+            if(is.null(weights)){
+              boot <- se_boot(data=data, formula=formula, n_x=n_x,
+                              n_samples=bootSamples[[1]],
+                              sample_size=bootSampleSize[[1]])
+            }
+            else{
+              boot <- se_boot(data=data, formula=formula, n_x=n_x,
+                              n_samples=bootSamples[[1]],
+                              sample_size=bootSampleSize[[1]],
+                              weights=weights)
+            }
 
             if(!is.null(boot)){
               ses_other <- cbind(ses_other, boot)
@@ -1118,12 +1162,20 @@ se_compare <- function(formula, data, types="all", cluster=NULL,
             samples <- rep(bootSamples, length(bootSampleSize))
             sample_sizes <- sort(rep(bootSampleSize, length(bootSamples)))
 
-            boot <- mapply(FUN=se_boot, n_samples=samples,
-                           sample_size=sample_sizes,
-                           MoreArgs=list(data=data, formula=formula, n_x=n_x))
+            if(is.null(weights)){
+              boot <- mapply(FUN=se_boot, n_samples=samples,
+                             sample_size=sample_sizes,
+                             MoreArgs=list(data=data, formula=formula, n_x=n_x))
+            }
+            else{
+              boot <- mapply(FUN=se_boot, n_samples=samples,
+                             sample_size=sample_sizes,
+                             MoreArgs=list(data=data, formula=formula, n_x=n_x,
+                                           weights=weights))
+            }
 
             if(!is.null(boot)){
-
+              print(boot)
               colnames(boot) <- paste("bootstrap_", "k", samples, "n",
                                       sample_sizes, "_FE", sep="")
 
@@ -1191,10 +1243,18 @@ se_compare <- function(formula, data, types="all", cluster=NULL,
     }
 
     # Estimate the non-FE model and get the coefficients
-    model <- lm(formula=as.formula(formula), data=data)
+    if(is.null(weights)){
+      model <- lm(formula=as.formula(formula), data=data)
+    }
+    else{
+      fmla <- as.formula(formula)
+      environment(fmla) <- environment()
+      model <- lm(formula=fmla, data=data, weights=get(weights))
+    }
 
     ses <- cbind(ses, matrix(model$coefficients, ncol=1,
-                             dimnames=list(c(names(model$coefficients)), c("estimate"))))
+                             dimnames=list(c(names(model$coefficients)),
+                                           c("estimate"))))
 
     # Parse the user's desired SE types
     if(!clusteredOnly){
@@ -1231,9 +1291,17 @@ se_compare <- function(formula, data, types="all", cluster=NULL,
           samples <- bootSamples
           sample_sizes <- bootSampleSize
 
-          boot <- se_boot(data=data, formula=formula, n_x=n_x,
-                          n_samples=bootSamples[[1]],
-                          sample_size=bootSampleSize[[1]])
+          if(is.null(weights)){
+            boot <- se_boot(data=data, formula=formula, n_x=n_x,
+                            n_samples=bootSamples[[1]],
+                            sample_size=bootSampleSize[[1]])
+          }
+          else{
+            boot <- se_boot(data=data, formula=formula, n_x=n_x,
+                            n_samples=bootSamples[[1]],
+                            sample_size=bootSampleSize[[1]],
+                            weights=(weights))
+          }
 
           if(!is.null(boot)){
             ses_other <- cbind(ses_other, boot)
@@ -1246,10 +1314,17 @@ se_compare <- function(formula, data, types="all", cluster=NULL,
         else{
           samples <- rep(bootSamples, length(bootSampleSize))
           sample_sizes <- sort(rep(bootSampleSize, length(bootSamples)))
-
-          boot <- mapply(FUN=se_boot, n_samples=samples,
-                         sample_size=sample_sizes,
-                         MoreArgs=list(data=data, formula=formula, n_x=n_x))
+          if(is.null(weights)){
+            boot <- mapply(FUN=se_boot, n_samples=samples,
+                           sample_size=sample_sizes,
+                           MoreArgs=list(data=data, formula=formula, n_x=n_x))
+          }
+          else{
+            boot <- mapply(FUN=se_boot, n_samples=samples,
+                           sample_size=sample_sizes,
+                           MoreArgs=list(data=data, formula=formula, n_x=n_x,
+                                         weights=weights))
+          }
 
           if(!is.null(boot)){
             colnames(boot) <- paste("bootstrap_", "k", samples, "n",
